@@ -5,7 +5,7 @@
 use std::io::Write;
 
 use crate::bindgen::declarationtyperesolver::DeclarationType;
-use crate::bindgen::ir::{Function, Type};
+use crate::bindgen::ir::{ConstExpr, Function, GenericArgument, Type};
 use crate::bindgen::writer::{ListType, SourceWriter};
 use crate::bindgen::{Config, Language};
 
@@ -35,7 +35,7 @@ impl CDeclarator {
 struct CDecl {
     type_qualifers: String,
     type_name: String,
-    type_generic_args: Vec<Type>,
+    type_generic_args: Vec<GenericArgument>,
     declarators: Vec<CDeclarator>,
     type_ctype: Option<DeclarationType>,
 }
@@ -51,29 +51,53 @@ impl CDecl {
         }
     }
 
-    fn from_type(t: &Type) -> CDecl {
+    fn from_type(t: &Type, config: &Config) -> CDecl {
         let mut cdecl = CDecl::new();
-        cdecl.build_type(t, false);
-        cdecl
-    }
-    fn from_func(f: &Function, layout_vertical: bool) -> CDecl {
-        let mut cdecl = CDecl::new();
-        cdecl.build_func(f, layout_vertical);
+        cdecl.build_type(t, false, config);
         cdecl
     }
 
-    fn build_func(&mut self, f: &Function, layout_vertical: bool) {
+    fn from_func_arg(t: &Type, array_length: Option<&str>, config: &Config) -> CDecl {
+        let mut cdecl = CDecl::new();
+        let length = match array_length {
+            Some(l) => l,
+            None => return CDecl::from_type(t, config),
+        };
+        let (ty, is_const) = match t {
+            Type::Ptr { ty, is_const, .. } => (ty, is_const),
+            _ => unreachable!(
+                "Should never have an array length for a non pointer type {:?}",
+                t
+            ),
+        };
+        let ptr_as_array = Type::Array(ty.clone(), ConstExpr::Value(length.to_string()));
+        cdecl.build_type(&ptr_as_array, *is_const, config);
+        cdecl
+    }
+
+    fn from_func(f: &Function, layout_vertical: bool, config: &Config) -> CDecl {
+        let mut cdecl = CDecl::new();
+        cdecl.build_func(f, layout_vertical, config);
+        cdecl
+    }
+
+    fn build_func(&mut self, f: &Function, layout_vertical: bool, config: &Config) {
         let args = f
             .args
             .iter()
-            .map(|arg| (arg.name.clone(), CDecl::from_type(&arg.ty)))
+            .map(|arg| {
+                (
+                    arg.name.clone(),
+                    CDecl::from_func_arg(&arg.ty, arg.array_length.as_deref(), config),
+                )
+            })
             .collect();
         self.declarators
             .push(CDeclarator::Func(args, layout_vertical));
-        self.build_type(&f.ret, false);
+        self.build_type(&f.ret, false, config);
     }
 
-    fn build_type(&mut self, t: &Type, is_const: bool) {
+    fn build_type(&mut self, t: &Type, is_const: bool, config: &Config) {
         match t {
             Type::Path(ref generic) => {
                 if is_const {
@@ -114,7 +138,7 @@ impl CDecl {
                     "error generating cdecl for {:?}",
                     t
                 );
-                self.type_name = p.to_string();
+                self.type_name = p.to_repr_c(config).to_string();
             }
             Type::Ptr {
                 ref ty,
@@ -127,17 +151,21 @@ impl CDecl {
                     is_nullable: *is_nullable,
                     is_ref: *is_ref,
                 });
-                self.build_type(ty, *ptr_is_const);
+                self.build_type(ty, *ptr_is_const, config);
             }
             Type::Array(ref t, ref constant) => {
                 let len = constant.as_str().to_owned();
                 self.declarators.push(CDeclarator::Array(len));
-                self.build_type(t, is_const);
+                self.build_type(t, is_const, config);
             }
-            Type::FuncPtr(ref ret, ref args) => {
+            Type::FuncPtr {
+                ref ret,
+                ref args,
+                is_nullable: _,
+            } => {
                 let args = args
                     .iter()
-                    .map(|(ref name, ref ty)| (name.clone(), CDecl::from_type(ty)))
+                    .map(|(ref name, ref ty)| (name.clone(), CDecl::from_type(ty, config)))
                     .collect();
                 self.declarators.push(CDeclarator::Ptr {
                     is_const: false,
@@ -145,7 +173,7 @@ impl CDecl {
                     is_ref: false,
                 });
                 self.declarators.push(CDeclarator::Func(args, false));
-                self.build_type(ret, false);
+                self.build_type(ret, false, config);
             }
         }
     }
@@ -156,8 +184,10 @@ impl CDecl {
             write!(out, "{} ", self.type_qualifers);
         }
 
-        if let Some(ref ctype) = self.type_ctype {
-            write!(out, "{} ", ctype.to_str());
+        if config.language != Language::Cython {
+            if let Some(ref ctype) = self.type_ctype {
+                write!(out, "{} ", ctype.to_str());
+            }
         }
 
         write!(out, "{}", self.type_name);
@@ -190,7 +220,7 @@ impl CDecl {
                     if is_const {
                         out.write("const ");
                     }
-                    if !is_nullable && !is_ref {
+                    if !is_nullable && !is_ref && config.language != Language::Cython {
                         if let Some(attr) = &config.pointer.non_null_attribute {
                             write!(out, "{} ", attr);
                         }
@@ -283,13 +313,13 @@ pub fn write_func<F: Write>(
     layout_vertical: bool,
     config: &Config,
 ) {
-    CDecl::from_func(f, layout_vertical).write(out, Some(f.path().name()), config);
+    CDecl::from_func(f, layout_vertical, config).write(out, Some(f.path().name()), config);
 }
 
 pub fn write_field<F: Write>(out: &mut SourceWriter<F>, t: &Type, ident: &str, config: &Config) {
-    CDecl::from_type(t).write(out, Some(ident), config);
+    CDecl::from_type(t, config).write(out, Some(ident), config);
 }
 
 pub fn write_type<F: Write>(out: &mut SourceWriter<F>, t: &Type, config: &Config) {
-    CDecl::from_type(t).write(out, None, config);
+    CDecl::from_type(t, config).write(out, None, config);
 }
