@@ -4,21 +4,22 @@
 
 use std::io::Write;
 
-use syn;
+use syn::ext::IdentExt;
 
 use crate::bindgen::config::{Config, Language, LayoutConfig};
 use crate::bindgen::declarationtyperesolver::DeclarationTypeResolver;
 use crate::bindgen::dependencies::Dependencies;
 use crate::bindgen::ir::{
-    AnnotationSet, Cfg, ConditionWrite, Constant, Documentation, GenericParams, Item,
-    ItemContainer, Path, Repr, ReprAlign, ReprStyle, ToCondition, Type, Typedef,
+    AnnotationSet, Cfg, ConditionWrite, Constant, Documentation, Field, GenericArgument,
+    GenericParams, Item, ItemContainer, Path, Repr, ReprAlign, ReprStyle, ToCondition, Type,
+    Typedef,
 };
 use crate::bindgen::library::Library;
 use crate::bindgen::mangle;
 use crate::bindgen::monomorph::Monomorphs;
 use crate::bindgen::rename::{IdentifierType, RenameRule};
 use crate::bindgen::reserved;
-use crate::bindgen::utilities::{find_first_some, IterHelpers};
+use crate::bindgen::utilities::IterHelpers;
 use crate::bindgen::writer::{ListType, Source, SourceWriter};
 
 #[derive(Debug, Clone)]
@@ -26,15 +27,14 @@ pub struct Struct {
     pub path: Path,
     pub export_name: String,
     pub generic_params: GenericParams,
-    pub fields: Vec<(String, Type, Documentation)>,
+    pub fields: Vec<Field>,
     /// Whether there's a tag field on the body of this struct. When this is
     /// true, is_enum_variant_body is also guaranteed to be true.
-    pub is_tagged: bool,
+    pub has_tag_field: bool,
     /// Whether this is an enum variant body.
     pub is_enum_variant_body: bool,
     pub alignment: Option<ReprAlign>,
     pub is_transparent: bool,
-    pub tuple_struct: bool,
     pub cfg: Option<Cfg>,
     pub annotations: AnnotationSet,
     pub documentation: Documentation,
@@ -44,7 +44,7 @@ pub struct Struct {
 impl Struct {
     /// Whether this struct can derive operator== / operator!=.
     pub fn can_derive_eq(&self) -> bool {
-        !self.fields.is_empty() && self.fields.iter().all(|x| x.1.can_cmp_eq())
+        !self.fields.is_empty() && self.fields.iter().all(|x| x.ty.can_cmp_eq())
     }
 
     pub fn add_associated_constant(&mut self, c: Constant) {
@@ -65,63 +65,65 @@ impl Struct {
             }
         };
 
-        let path = Path::new(item.ident.to_string());
+        let path = Path::new(item.ident.unraw().to_string());
 
         // Ensure we can safely represent the struct given the configuration.
         if let Some(align) = repr.align {
             layout_config.ensure_safe_to_represent(&align)?;
         }
 
-        let (fields, tuple_struct) = match item.fields {
-            syn::Fields::Unit => (Vec::new(), false),
-            syn::Fields::Named(ref fields) => {
-                let out = fields
-                    .named
-                    .iter()
-                    .try_skip_map(|x| x.as_ident_and_type(&path))?;
-                (out, false)
-            }
+        let fields = match item.fields {
+            syn::Fields::Unit => Vec::new(),
+            syn::Fields::Named(ref fields) => fields
+                .named
+                .iter()
+                .try_skip_map(|field| Field::load(field, &path))?,
             syn::Fields::Unnamed(ref fields) => {
                 let mut out = Vec::new();
                 let mut current = 0;
                 for field in fields.unnamed.iter() {
-                    if let Some(mut x) = Type::load(&field.ty)? {
-                        x.replace_self_with(&path);
-                        out.push((format!("{}", current), x, Documentation::load(&field.attrs)));
+                    if let Some(mut ty) = Type::load(&field.ty)? {
+                        ty.replace_self_with(&path);
+                        out.push(Field {
+                            name: format!("{}", current),
+                            ty,
+                            cfg: Cfg::load(&field.attrs),
+                            annotations: AnnotationSet::load(&field.attrs)?,
+                            documentation: Documentation::load(&field.attrs),
+                        });
                         current += 1;
                     }
                 }
-                (out, true)
+                out
             }
         };
 
-        let is_tagged = false;
+        let has_tag_field = false;
         let is_enum_variant_body = false;
 
         Ok(Struct::new(
             path,
-            GenericParams::new(&item.generics),
+            GenericParams::load(&item.generics)?,
             fields,
-            is_tagged,
+            has_tag_field,
             is_enum_variant_body,
             repr.align,
             is_transparent,
-            tuple_struct,
             Cfg::append(mod_cfg, Cfg::load(&item.attrs)),
             AnnotationSet::load(&item.attrs)?,
             Documentation::load(&item.attrs),
         ))
     }
 
+    #[allow(clippy::too_many_arguments)]
     pub fn new(
         path: Path,
         generic_params: GenericParams,
-        fields: Vec<(String, Type, Documentation)>,
-        is_tagged: bool,
+        fields: Vec<Field>,
+        has_tag_field: bool,
         is_enum_variant_body: bool,
         alignment: Option<ReprAlign>,
         is_transparent: bool,
-        tuple_struct: bool,
         cfg: Option<Cfg>,
         annotations: AnnotationSet,
         documentation: Documentation,
@@ -132,11 +134,10 @@ impl Struct {
             export_name,
             generic_params,
             fields,
-            is_tagged,
+            has_tag_field,
             is_enum_variant_body,
             alignment,
             is_transparent,
-            tuple_struct,
             cfg,
             annotations,
             documentation,
@@ -144,9 +145,9 @@ impl Struct {
         }
     }
 
-    pub fn simplify_standard_types(&mut self) {
-        for &mut (_, ref mut ty, _) in &mut self.fields {
-            ty.simplify_standard_types();
+    pub fn simplify_standard_types(&mut self, config: &Config) {
+        for field in &mut self.fields {
+            field.ty.simplify_standard_types(config);
         }
     }
 
@@ -161,40 +162,41 @@ impl Struct {
             return;
         }
 
-        for &(_, ref ty, _) in &self.fields {
-            ty.add_monomorphs(library, out);
+        for field in &self.fields {
+            field.ty.add_monomorphs(library, out);
         }
     }
 
     pub fn mangle_paths(&mut self, monomorphs: &Monomorphs) {
-        for &mut (_, ref mut ty, _) in &mut self.fields {
-            ty.mangle_paths(monomorphs);
+        for field in &mut self.fields {
+            field.ty.mangle_paths(monomorphs);
         }
     }
 
     pub fn specialize(
         &self,
-        generic_values: &[Type],
-        mappings: &[(&Path, &Type)],
+        generic_values: &[GenericArgument],
+        mappings: &[(&Path, &GenericArgument)],
         config: &Config,
     ) -> Self {
-        let mangled_path = mangle::mangle_path(
-            &self.path,
-            generic_values,
-            config.export.mangle_separator.as_deref(),
-        );
+        let mangled_path = mangle::mangle_path(&self.path, generic_values, &config.export.mangle);
         Struct::new(
             mangled_path,
             GenericParams::default(),
             self.fields
                 .iter()
-                .map(|x| (x.0.clone(), x.1.specialize(mappings), x.2.clone()))
+                .map(|field| Field {
+                    name: field.name.clone(),
+                    ty: field.ty.specialize(mappings),
+                    cfg: field.cfg.clone(),
+                    annotations: field.annotations.clone(),
+                    documentation: field.documentation.clone(),
+                })
                 .collect(),
-            self.is_tagged,
+            self.has_tag_field,
             self.is_enum_variant_body,
             self.alignment,
             self.is_transparent,
-            self.tuple_struct,
             self.cfg.clone(),
             self.annotations.clone(),
             self.documentation.clone(),
@@ -203,6 +205,7 @@ impl Struct {
 
     fn emit_bitflags_binop<F: Write>(
         &self,
+        constexpr_prefix: &str,
         operator: char,
         other: &str,
         out: &mut SourceWriter<F>,
@@ -210,7 +213,8 @@ impl Struct {
         out.new_line();
         write!(
             out,
-            "{} operator{}(const {}& {}) const",
+            "{}{} operator{}(const {}& {}) const",
+            constexpr_prefix,
             self.export_name(),
             operator,
             self.export_name(),
@@ -219,8 +223,10 @@ impl Struct {
         out.open_brace();
         write!(
             out,
-            "return {{static_cast<decltype(bits)>(this->bits {} {}.bits)}};",
-            operator, other
+            "return {} {{ static_cast<decltype(bits)>(this->bits {} {}.bits) }};",
+            self.export_name(),
+            operator,
+            other
         );
         out.close_brace(false);
 
@@ -267,20 +273,22 @@ impl Item for Struct {
     }
 
     fn collect_declaration_types(&self, resolver: &mut DeclarationTypeResolver) {
-        if !self.is_transparent {
+        if self.is_transparent {
+            resolver.add_none(&self.path);
+        } else {
             resolver.add_struct(&self.path);
         }
     }
 
     fn resolve_declaration_types(&mut self, resolver: &DeclarationTypeResolver) {
-        for &mut (_, ref mut ty, _) in &mut self.fields {
-            ty.resolve_declaration_types(resolver);
+        for field in &mut self.fields {
+            field.ty.resolve_declaration_types(resolver);
         }
     }
 
     fn rename_for_config(&mut self, config: &Config) {
         // Rename the name of the struct
-        if !self.is_tagged || config.language == Language::C {
+        if !(self.has_tag_field && config.language == Language::Cxx) {
             config.export.rename(&mut self.export_name);
         }
 
@@ -289,9 +297,9 @@ impl Item for Struct {
             let fields = self
                 .fields
                 .iter_mut()
-                .skip(if self.is_tagged { 1 } else { 0 });
-            for &mut (_, ref mut ty, _) in fields {
-                ty.rename_for_config(config, &self.generic_params);
+                .skip(if self.has_tag_field { 1 } else { 0 });
+            for field in fields {
+                field.ty.rename_for_config(config, &self.generic_params);
             }
         }
 
@@ -306,37 +314,34 @@ impl Item for Struct {
 
         // Scope for mutable borrow of fields
         {
-            let mut names = self.fields.iter_mut().map(|field| &mut field.0);
+            let names = self.fields.iter_mut().map(|field| &mut field.name);
 
-            let field_rules = [
-                self.annotations.parse_atom::<RenameRule>("rename-all"),
-                config.structure.rename_fields,
-            ];
+            let field_rules = self
+                .annotations
+                .parse_atom::<RenameRule>("rename-all")
+                .unwrap_or(config.structure.rename_fields);
 
             if let Some(o) = self.annotations.list("field-names") {
                 for (dest, src) in names.zip(o) {
                     *dest = src;
                 }
-            } else if let Some(r) = find_first_some(&field_rules) {
+            } else if let Some(r) = field_rules.not_none() {
                 for name in names {
-                    *name = r.apply(name, IdentifierType::StructMember);
+                    *name = r.apply(name, IdentifierType::StructMember).into_owned();
                 }
-            } else if self.tuple_struct {
-                // If there is a tag field, skip it
-                if self.is_tagged {
-                    names.next();
-                }
-
+            } else {
                 // If we don't have any rules for a tuple struct, prefix them with
-                // an underscore so it still compiles
+                // an underscore so it still compiles.
                 for name in names {
-                    name.insert(0, '_');
+                    if name.starts_with(|c: char| c.is_ascii_digit()) {
+                        name.insert(0, '_');
+                    }
                 }
             }
         }
 
         for field in &mut self.fields {
-            reserved::escape(&mut field.0);
+            reserved::escape(&mut field.name);
         }
 
         for c in self.associated_constants.iter_mut() {
@@ -348,12 +353,14 @@ impl Item for Struct {
         let mut fields = self.fields.iter();
 
         // If there is a tag field, skip it
-        if self.is_tagged {
+        if self.has_tag_field {
             fields.next();
         }
 
-        for &(_, ref ty, _) in fields {
-            ty.add_dependencies_ignoring_generics(&self.generic_params, library, out);
+        for field in fields {
+            field
+                .ty
+                .add_dependencies_ignoring_generics(&self.generic_params, library, out);
         }
 
         for c in &self.associated_constants {
@@ -363,35 +370,13 @@ impl Item for Struct {
 
     fn instantiate_monomorph(
         &self,
-        generic_values: &[Type],
+        generic_values: &[GenericArgument],
         library: &Library,
         out: &mut Monomorphs,
     ) {
-        assert!(
-            self.generic_params.len() > 0,
-            "{} is not generic",
-            self.path
-        );
-        assert!(
-            self.generic_params.len() == generic_values.len(),
-            "{} has {} params but is being instantiated with {} values",
-            self.path,
-            self.generic_params.len(),
-            generic_values.len(),
-        );
-
-        let mappings = self
-            .generic_params
-            .iter()
-            .zip(generic_values.iter())
-            .collect::<Vec<_>>();
-
+        let mappings = self.generic_params.call(self.path.name(), generic_values);
         let monomorph = self.specialize(generic_values, &mappings, library.get_config());
-
-        // Instantiate any monomorphs for any generic paths we may have just created.
-        monomorph.add_monomorphs(library, out);
-
-        out.insert_struct(self, monomorph, generic_values.to_owned());
+        out.insert_struct(library, self, monomorph, generic_values.to_owned());
     }
 }
 
@@ -402,7 +387,7 @@ impl Source for Struct {
                 path: self.path.clone(),
                 export_name: self.export_name.to_owned(),
                 generic_params: self.generic_params.clone(),
-                aliased: self.fields[0].1.clone(),
+                aliased: self.fields[0].ty.clone(),
                 cfg: self.cfg.clone(),
                 annotations: self.annotations.clone(),
                 documentation: self.documentation.clone(),
@@ -431,34 +416,50 @@ impl Source for Struct {
         //   typedef struct {
         // C with Both as style:
         //   typedef struct Name {
-        if config.language == Language::C && config.style.generate_typedef() {
-            out.write("typedef ");
+        match config.language {
+            Language::C if config.style.generate_typedef() => out.write("typedef "),
+            Language::C | Language::Cxx => {}
+            Language::Cython => out.write(config.style.cython_def()),
+        }
+
+        // Cython extern declarations don't manage layouts, layouts are defined entierly by the
+        // corresponding C code. So this `packed` is only for documentation, and missing
+        // `aligned(n)` is also not a problem.
+        if config.language == Language::Cython {
+            if let Some(align) = self.alignment {
+                match align {
+                    ReprAlign::Packed => out.write("packed "),
+                    ReprAlign::Align(_) => {} // Not supported
+                }
+            }
         }
 
         out.write("struct");
 
-        if let Some(align) = self.alignment {
-            match align {
-                ReprAlign::Packed => {
-                    if let Some(ref anno) = config.layout.packed {
-                        write!(out, " {}", anno);
+        if config.language != Language::Cython {
+            if let Some(align) = self.alignment {
+                match align {
+                    ReprAlign::Packed => {
+                        if let Some(ref anno) = config.layout.packed {
+                            write!(out, " {}", anno);
+                        }
                     }
-                }
-                ReprAlign::Align(n) => {
-                    if let Some(ref anno) = config.layout.aligned_n {
-                        write!(out, " {}({})", anno, n);
+                    ReprAlign::Align(n) => {
+                        if let Some(ref anno) = config.layout.aligned_n {
+                            write!(out, " {}({})", anno, n);
+                        }
                     }
                 }
             }
         }
 
-        if self.annotations.must_use {
+        if self.annotations.must_use(config) {
             if let Some(ref anno) = config.structure.must_use {
                 write!(out, " {}", anno);
             }
         }
 
-        if config.language == Language::Cxx || config.style.generate_tag() {
+        if config.language != Language::C || config.style.generate_tag() {
             write!(out, " {}", self.export_name());
         }
 
@@ -470,15 +471,9 @@ impl Source for Struct {
             out.new_line();
         }
 
-        if config.documentation {
-            out.write_vertical_source_list(&self.fields, ListType::Cap(";"));
-        } else {
-            let vec: Vec<_> = self
-                .fields
-                .iter()
-                .map(|&(ref name, ref ty, _)| (name.clone(), ty.clone()))
-                .collect();
-            out.write_vertical_source_list(&vec[..], ListType::Cap(";"));
+        out.write_vertical_source_list(&self.fields, ListType::Cap(";"));
+        if config.language == Language::Cython && self.fields.is_empty() {
+            out.write("pass");
         }
 
         if config.language == Language::Cxx {
@@ -496,17 +491,19 @@ impl Source for Struct {
                     config
                         .function
                         .rename_args
-                        .as_ref()
-                        .unwrap_or(&RenameRule::GeckoCase)
                         .apply(name, IdentifierType::FunctionArg)
+                        .into_owned()
                 };
                 write!(out, "{}(", self.export_name());
                 let vec: Vec<_> = self
                     .fields
                     .iter()
-                    .map(|&(ref name, ref ty, _)| {
-                        // const-ref args to constructor
-                        (format!("const& {}", arg_renamer(name)), ty.clone())
+                    .map(|field| {
+                        Field::from_name_and_type(
+                            // const-ref args to constructor
+                            format!("const& {}", arg_renamer(&field.name)),
+                            field.ty.clone(),
+                        )
                     })
                     .collect();
                 out.write_vertical_source_list(&vec[..], ListType::Join(","));
@@ -516,7 +513,7 @@ impl Source for Struct {
                 let vec: Vec<_> = self
                     .fields
                     .iter()
-                    .map(|x| format!("{}({})", x.0, arg_renamer(&x.0)))
+                    .map(|field| format!("{}({})", field.name, arg_renamer(&field.name)))
                     .collect();
                 out.write_vertical_source_list(&vec[..], ListType::Join(","));
                 out.new_line();
@@ -524,11 +521,10 @@ impl Source for Struct {
                 out.new_line();
             }
 
-            let other = if let Some(r) = config.function.rename_args {
-                r.apply("other", IdentifierType::FunctionArg)
-            } else {
-                String::from("other")
-            };
+            let other = config
+                .function
+                .rename_args
+                .apply("other", IdentifierType::FunctionArg);
 
             if self
                 .annotations
@@ -539,24 +535,76 @@ impl Source for Struct {
                     wrote_start_newline = true;
                     out.new_line();
                 }
+                let constexpr_prefix = if config.constant.allow_constexpr {
+                    "constexpr "
+                } else {
+                    ""
+                };
+
                 out.new_line();
-                write!(out, "explicit operator bool() const");
+                write!(out, "{}explicit operator bool() const", constexpr_prefix);
                 out.open_brace();
                 write!(out, "return !!bits;");
                 out.close_brace(false);
 
                 out.new_line();
-                write!(out, "{} operator~() const", self.export_name());
+                write!(
+                    out,
+                    "{}{} operator~() const",
+                    constexpr_prefix,
+                    self.export_name()
+                );
                 out.open_brace();
-                write!(out, "return {{static_cast<decltype(bits)>(~bits)}};");
+                write!(
+                    out,
+                    "return {} {{ static_cast<decltype(bits)>(~bits) }};",
+                    self.export_name()
+                );
                 out.close_brace(false);
-
-                self.emit_bitflags_binop('|', &other, out);
-                self.emit_bitflags_binop('&', &other, out);
-                self.emit_bitflags_binop('^', &other, out);
+                self.emit_bitflags_binop(constexpr_prefix, '|', &other, out);
+                self.emit_bitflags_binop(constexpr_prefix, '&', &other, out);
+                self.emit_bitflags_binop(constexpr_prefix, '^', &other, out);
             }
 
-            let skip_fields = if self.is_tagged { 1 } else { 0 };
+            // Generate a serializer function that allows dumping this struct
+            // to an std::ostream. It's defined as a friend function inside the
+            // struct definition, and doesn't need the `inline` keyword even
+            // though it's implemented right in the generated header file.
+            if config.structure.derive_ostream(&self.annotations) {
+                if !wrote_start_newline {
+                    wrote_start_newline = true;
+                    out.new_line();
+                }
+
+                out.new_line();
+                let stream = config
+                    .function
+                    .rename_args
+                    .apply("stream", IdentifierType::FunctionArg);
+                let instance = config
+                    .function
+                    .rename_args
+                    .apply("instance", IdentifierType::FunctionArg);
+                write!(
+                    out,
+                    "friend std::ostream& operator<<(std::ostream& {}, const {}& {})",
+                    stream,
+                    self.export_name(),
+                    instance,
+                );
+                out.open_brace();
+                write!(out, "return {} << \"{{ \"", stream);
+                let vec: Vec<_> = self
+                    .fields
+                    .iter()
+                    .map(|x| format!(" << \"{}=\" << {}.{}", x.name, instance, x.name))
+                    .collect();
+                out.write_vertical_source_list(&vec[..], ListType::Join(" << \", \""));
+                out.write(" << \" }\";");
+                out.close_brace(false);
+            }
+
+            let skip_fields = if self.has_tag_field { 1 } else { 0 };
 
             macro_rules! emit_op {
                 ($op_name:expr, $op:expr, $conjuc:expr) => {{
@@ -589,7 +637,7 @@ impl Source for Struct {
                         .fields
                         .iter()
                         .skip(skip_fields)
-                        .map(|x| format!("{} {} {}.{}", x.0, $op, other, x.0))
+                        .map(|field| format!("{} {} {}.{}", field.name, $op, other, field.name))
                         .collect();
                     out.write_vertical_source_list(
                         &vec[..],
@@ -598,7 +646,7 @@ impl Source for Struct {
                     out.write(";");
                     out.close_brace(false);
                 }};
-            };
+            }
 
             if config.structure.derive_eq(&self.annotations) && self.can_derive_eq() {
                 emit_op!("eq", "==", "&&");
@@ -608,25 +656,25 @@ impl Source for Struct {
             }
             if config.structure.derive_lt(&self.annotations)
                 && self.fields.len() == 1
-                && self.fields[0].1.can_cmp_order()
+                && self.fields[0].ty.can_cmp_order()
             {
                 emit_op!("lt", "<", "&&");
             }
             if config.structure.derive_lte(&self.annotations)
                 && self.fields.len() == 1
-                && self.fields[0].1.can_cmp_order()
+                && self.fields[0].ty.can_cmp_order()
             {
                 emit_op!("lte", "<=", "&&");
             }
             if config.structure.derive_gt(&self.annotations)
                 && self.fields.len() == 1
-                && self.fields[0].1.can_cmp_order()
+                && self.fields[0].ty.can_cmp_order()
             {
                 emit_op!("gt", ">", "&&");
             }
             if config.structure.derive_gte(&self.annotations)
                 && self.fields.len() == 1
-                && self.fields[0].1.can_cmp_order()
+                && self.fields[0].ty.can_cmp_order()
             {
                 emit_op!("gte", ">=", "&&");
             }
@@ -663,35 +711,3 @@ impl Source for Struct {
         condition.write_after(config, out);
     }
 }
-
-pub trait SynFieldHelpers {
-    fn as_ident_and_type(
-        &self,
-        self_path: &Path,
-    ) -> Result<Option<(String, Type, Documentation)>, String>;
-}
-
-impl SynFieldHelpers for syn::Field {
-    fn as_ident_and_type(
-        &self,
-        self_path: &Path,
-    ) -> Result<Option<(String, Type, Documentation)>, String> {
-        let ident = self
-            .ident
-            .as_ref()
-            .ok_or_else(|| "field is missing identifier".to_string())?
-            .clone();
-        let converted_ty = Type::load(&self.ty)?;
-
-        if let Some(mut x) = converted_ty {
-            x.replace_self_with(self_path);
-            Ok(Some((
-                ident.to_string(),
-                x,
-                Documentation::load(&self.attrs),
-            )))
-        } else {
-            Ok(None)
-        }
-    }
-}
